-- ****************************************
-- **		 GENERIC FUNCTIONS		  **
-- **									**
-- ** Copyright (c) 2017 SpinalSoft (AU) **
-- ****************************************
-- Check that the incoming Text is valid and
-- ONLY contains Alphanumeric Characters and Underscore
function checkTextInput ( inText )
	if ( not inText or inText == "" ) then
		return false
	end

	if ( inText:match ( "%W" ) ) then
		if ( not inText:match ( "_" ) ) then
			return false
		end
	end

	return true
end

-- This Function is identical to the above
-- BUT allows SPACES as well
function checkTextInputS ( inText )
	if ( not inText or inText == "" ) then
		return false
	end

	if ( inText:match ( "%W" ) ) then
		if ( not inText:match ( "_" ) and not inText:match ( " " ) ) then
			return false
		end
	end

	return true
end

-- A simple Bubble Sort routine for a 1-Dimensional Lua Table
function bubbleSort ( inTbl )
	local itemCount = getTableLength ( inTbl )
	local hasChanged

	repeat
		hasChanged = false
		itemCount = itemCount - 1
 
		for i = 1, itemCount do
			if inTbl[i] > inTbl[i + 1] then
				inTbl[i], inTbl[i + 1] = inTbl[i + 1], inTbl[i]
				hasChanged = true
			end
		end
	until ( hasChanged == false )

	return inTbl
end

-- A simple copy routine for Lua Tables
-- makes a straight 1->1 copy
function shallowCopy ( inTbl )
	local t = { }

	for k, v in pairs ( inTbl ) do
		t[k] = v
	end

	return t
end

-- A slightly different copy routine for Lua Tables
-- ignores any Keys
function shallowCopy2 ( inTbl )
	local t = { }

	for k, v in pairs ( inTbl ) do
		table.insert ( t, v )
	end

	return t
end

function quickSort ( inTbl, iStart, iEnd )
	iStart = iStart or 1
	iEnd = iEnd or getTableLength ( inTbl )

	if ( iEnd - iStart < 1 ) then
		return inTbl
	end

	local pivot = iStart

	for x = iStart + 1, iEnd do
		if inTbl[x] <= inTbl[pivot] then
			local sTemp = inTbl[pivot + 1]
			inTbl[pivot + 1] = inTbl[pivot]

			if ( x == pivot + 1 ) then
				inTbl[pivot] = sTemp
			else
				inTbl[pivot] = inTbl[x]
				inTbl[x] = sTemp
			end

			pivot = pivot + 1
		end
	end

	inTbl = quickSort ( inTbl, iStart, pivot - 1 )

	return quickSort ( inTbl, pivot + 1, iEnd )
end

-- Remove Duplicate values from a 1-Dimensional Lua Table
function removeDuplicates ( inTbl )
	local tHash = { }
	local tRes = { }
	local iTR = 0

	for _, v in pairs ( inTbl ) do
		if ( not tHash[v] ) then
			tRes[iTR + 1] = v
			tHash[v] = true
			iTR = iTR + 1
		end
	end

	return tRes
end

-- Converts First character of String to Upper Case
function firstToUpper ( inStr )
	return ( inStr:gsub ( "^%l", string.upper ) )
end

-- Accurately get the Length of a 1-Dimensional Lua Table
function getTableLength ( inTbl )
	local tblCount = 0

	for _ in pairs ( inTbl ) do
		tblCount = tblCount + 1
	end

	return tblCount
end

-- Empty a Qt Table (NOT a Lua Table!!)
function emptyTable ( inTbl )
	for x = qt.getNumTableRows ( thisEditor, inTbl ), 1, -1 do
		qt.removeTableRow ( thisEditor, inTbl, x - 1 )
	end
end

-- Sort a Qt Table (NOT a Lua Table!!)
-- inSize determines the number of entries in the Table
-- NB: This only allows for a MAXIMUM of 5
function sortTable ( inTbl, inSize )
	inSize = inSize or 1
	local toString = tostring
	local tT = { }
	local tT1 = { }

	for x = 1, qt.getNumTableRows ( thisEditor, inTbl ) do
		tT1 = { }

		for y = 1, inSize do
			sTmp = qt.getTableItemText ( thisEditor, inTbl, x - 1, y - 1 )
			tT1[y] = sTmp
		end

		tT[x] = tT1
	end

	table.sort ( tT, function ( a, b )
		if ( a[1] < b[1] ) then
			return true
		elseif ( a[1] > b[1] ) then
			return false
		else
			if ( a[2] < b[2] ) then
				return true
			elseif ( a[2] > b[2] ) then
				return false
			else
				if ( a[3] < b[3] ) then
					return true
				elseif ( a[3] > b[3] ) then
					return false
				else
					if ( a[4] < b[4] ) then
						return true
					elseif ( a[4] > b[4] ) then
						return false
					else
						return a[5] < b[5]
					end
				end
			end
		end
	end)

	emptyTable ( inTbl )

	for x = 1, getTableLength ( tT ) do
		qt.insertTableRow ( thisEditor, inTbl, x - 1 )
		qt.setTableItemText ( thisEditor, inTbl, x - 1, 0, tT[x][1] )
		qt.setTableItemTextAlign ( thisEditor, inTbl, x - 1, 0, "center" )

		if ( inSize > 1 ) then
			qt.setTableItemText ( thisEditor, inTbl, x - 1, 1, tT[x][2] )
			qt.setTableItemTextAlign ( thisEditor, inTbl, x - 1, 1, "center" )

			if ( inSize > 2 ) then
				qt.setTableItemText ( thisEditor, inTbl, x - 1, 2, tT[x][3] )
				qt.setTableItemTextAlign ( thisEditor, inTbl, x - 1, 2, "center" )

				if ( inSize > 3 ) then
					qt.setTableItemText ( thisEditor, inTbl, x - 1, 3, tT[x][4] )
					qt.setTableItemTextAlign ( thisEditor, inTbl, x - 1, 3, "center" )

					if ( inSize > 4 ) then
						qt.setTableItemText ( thisEditor, inTbl, x - 1, 4, tT[x][5] )
						qt.setTableItemTextAlign ( thisEditor, inTbl, x - 1, 4, "center" )
					end
				end
			end
		end
	end
end

-- Gets all Files in a Directory and returns them in a Table
function getFilesInDirectory ( filePath, prepend_path_to_filenames, fileType )
	if filePath:sub ( -1 ) ~= '\\' then
		filePath = filePath .. '\\'
	end

	local pipe = io.popen ( 'dir /b/a-d "' .. filePath .. '*.*" 2> nul' )
	local output = pipe:read'*a'

	pipe:close ( )

	local tFiles = { }

	for fileName in output:gmatch'[^\r\n]+' do
		if prepend_path_to_filenames then
			fileName = filePath .. fileName
		end

		-- ONLY SAVE FILES WITH TYPE MATCHING fileType (i.e ".cha" etc.)
		if ( string.sub ( fileName, -4, -1 ) == fileType ) then
			table.insert ( tFiles, fileName )
		end
	end

	return tFiles
end

-- This shows a generic Dialog Box
-- inType = 1 - Question Dialog with Yes/No Buttons
-- inType = 2 - Information Dialog with OK Button
-- inType = 3 - Error Dialog with OK Button
-- inType = 4 - Warning Dialog with Yes/No Buttons
function showDialog ( inType, inText )
	qt.openEditor ( thisEditor, "Dialog", 1 )
	qt.setText ( thisEditor, "txtDlgText", inText )

	if ( inType == 1 or inType == 4 ) then
		if ( inType == 1 ) then
			qt.setPixmap ( thisEditor, "txtDlgImg", "question.png" )
		else
			qt.setPixmap ( thisEditor, "txtDlgImg", "warning.png" )
		end

		qt.showWidget ( thisEditor, "btnDlgRight" )
		qt.setText ( thisEditor, "btnDlgCenter", "Yes" )
		qt.setText ( thisEditor, "btnDlgRight", "No" )
	else
		if ( inType == 2 ) then
			qt.setPixmap ( thisEditor, "txtDlgImg", "info.png" )
		else
			qt.setPixmap ( thisEditor, "txtDlgImg", "error.png" )
		end

		qt.hideWidget ( thisEditor, "btnDlgRight" )
		qt.setText ( thisEditor, "btnDlgCenter", "OK" )
	end
end

-- Check if a value is a Table
function isTable ( t )
	return type ( t ) == "table"
end

-- Print a LUA Table to the Skyline Console
function print_r ( t )
	local print_r_cache= { };

	local function sub_print_r ( t, indent )
		if ( print_r_cache[tostring ( t )] ) then
			sky.lprint ( indent .. "*" .. tostring ( t ) );
		else
			print_r_cache[tostring ( t )] = true;

			if ( type ( t ) == "table" ) then
				for pos, val in pairs ( t ) do
					if ( type ( val ) == "table" ) then
						sky.lprint ( indent .. "[" .. pos .. "] => " .. tostring ( t ) .. " {" );
						sub_print_r ( val, indent .. string.rep ( " ", string.len ( pos ) + 8 ) );
						sky.lprint ( indent .. string.rep ( " ", string.len ( pos ) + 6 ) .. "}" );
					elseif ( type ( val ) == "string" ) then
						sky.lprint ( indent .. "[" .. pos .. '] => "' .. val .. '"' );
					else
						sky.lprint ( indent .. "[" .. pos .. "] => " .. tostring ( val ) );
					end
				end
			else
				sky.lprint ( indent .. tostring ( t ) );
			end
		end
	end

	if ( type ( t ) == "table" ) then
		sky.lprint ( tostring ( t ) .. " {" );
		sub_print_r ( t, "  " );
		sky.lprint ( "}" );
	else
		sub_print_r ( t, "  " );
	end

	sky.lprint ( "" );
end

