--[[
	Copyright (c) 2013 David Young dayoung@goliathdesigns.com

	This software is provided 'as-is', without any express or implied
	warranty. In no event will the authors be held liable for any damages
	arising from the use of this software.

	Permission is granted to anyone to use this software for any purpose,
	including commercial applications, and to alter it and redistribute it
	freely, subject to the following restrictions:

	1. The origin of this software must not be misrepresented you must not
	claim that you wrote the original software. If you use this software
	in a product, an acknowledgement in the product documentation would be
	appreciated but is not required.

	2. Altered source versions must be plainly marked as such, and must not be
	misrepresented as being the original software.

	3. This notice may not be removed or altered from any source
	distribution.

**** Code rewritten for use in the Skyline Game Engine by SpinalSoft (AU) 2017 ****
]]

sky.include "behaviortreenode.lua"

function btNew ( objID )
	local self = {
		bt_currentNode_,
		bt_node_,
		bt_objID_ = objID
	}

	local bt_evaluateSelector = function ( self, inNode, deltaTimeInMillis )
		for index = 1, #inNode.btn_children_ do
			local tmpChild = inNode:BTN_GetChild ( index )

			if ( tmpChild.btn_type_ == "ACTION" ) then
				return { node = tmpChild, result = true }
			elseif ( tmpChild.btn_type_ == "CONDITION" ) then
				sky.lprint ( "BEHAVIOR TREE ERROR - child.type_ cannot be a Condition" )
				return { result = false }
			elseif ( tmpChild.btn_type_ == "SELECTOR" ) then
				local result = bt_evaluateSelector ( self, tmpChild, deltaTimeInMillis )

				if ( result.result ) then
					return result
				end
			elseif ( tmpChild.btn_type_ == "SEQUENCE" ) then
				local result = bt_evaluateSequence ( self, tmpChild, deltaTimeInMillis )

				if ( result.result ) then
					return result
				end
			end
		end

		return { result = false }
	end

	bt_evaluateSequence = function ( self, inNode, deltaTimeInMillis, index )
		index = index or 1

		for count = index, #inNode.btn_children_ do
			local tmpChild = inNode:BTN_GetChild ( count )

			if ( tmpChild.btn_type_ == "ACTION" ) then
				return { node = tmpChild, result = true }
			elseif ( tmpChild.btn_type_ == "CONDITION" ) then
				local result = tmpChild.btn_evaluator_ ( self.bt_objID_ )

				if ( not result ) then
					return { result = false }
				end
			elseif ( tmpChild.btn_type_ == "SELECTOR" ) then
				local result = bt_evaluateSelector ( self, tmpChild, deltaTimeInMillis )

				if ( not result.result ) then
					return { result = false }
				elseif ( result.node ~= nil ) then
					return result
				end
			elseif ( tmpChild.btn_type_ == "SEQUENCE" ) then
				local result = bt_evaluateSequence ( self, tmpChild, deltaTimeInMillis )

				if ( not result.result ) then
					return { result = false }
				elseif ( result.node ~= nil ) then
					return result
				end
			end

			count = count + 1
		end

		return { result = true }
	end

	local bt_evaluateNode = function ( self, inNode, deltaTimeInMillis )
		if ( inNode.btn_type_ == "ACTION" ) then
			return inNode
		elseif ( inNode.btn_type_ == "CONDITION" ) then
			sky.lprint ( "BEHAVIOR TREE ERROR - conditions MUST be evaluated immediately" )
		elseif ( inNode.btn_type_ == "SELECTOR" ) then
			local result = bt_evaluateSelector ( self, inNode, deltaTimeInMillis )

			if ( result.result ) then
				return result.node
			end
		elseif ( inNode.btn_type_ == "SEQUENCE" ) then
			local result = bt_evaluateSequence ( self, inNode, deltaTimeInMillis )

			if ( result.result ) then
				return result.node
			end
		end
	end

	local bt_continueEvaluation = function ( self, inNode, deltaTimeInMillis )
		local parentNode = inNode:BTN_GetParent ( )
		local childNode = inNode

		while ( parentNode ~= nil ) do
			if ( parentNode.btn_type_ == "SEQUENCE" ) then
				local childIndex = parentNode:BTN_ChildIndex ( childNode )

				if ( childIndex < parentNode:BTN_GetNumberOfChildren ( ) ) then
					return bt_evaluateSequence ( self, parentNode, deltaTimeInMillis, childIndex + 1 )
				end
			end

			childNode = parentNode
			parentNode = childNode:BTN_GetParent ( )
		end
	end

	local bt_update = function ( self, deltaTimeInMillis )
		if ( self.bt_currentNode_ == nil ) then
			self.bt_currentNode_ = bt_evaluateNode ( self, self.bt_node_, deltaTimeInMillis )
		else
			local status = self.bt_currentNode_.btn_action_.act_status_

			if ( status == "UNINITIALIZED" ) then
				self.bt_currentNode_.btn_action_:ACT_Initialize ( self.bt_objID_ )
				self.bt_currentNode_.btn_action_.act_status_ = "RUNNING"
			elseif ( status == "TERMINATED" ) then
				self.bt_currentNode_.btn_action_:ACT_CleanUp ( self.bt_objID_ )
				self.bt_currentNode_ = bt_continueEvaluation ( self.bt_objID_, self.bt_currentNode_, deltaTimeInMillis )
			elseif ( status == "RUNNING" ) then
				self.bt_currentNode_.btn_action_.act_status_ = self.bt_currentNode_.btn_action_:ACT_Update ( self.bt_objID_, deltaTimeInMillis )
			end
		end
	end

	local bt_createSelector = function ( )
		return btnNew ( "selector", "SELECTOR" )
	end

	local bt_createSequence = function ( )
		return btnNew ( "sequence", "SEQUENCE" )
	end

	local bt_createAction = function ( name, action )
		local tmpNode = btnNew ( name, "ACTION")
		tmpNode:BTN_SetAction ( action )

		return tmpNode
	end

	local bt_createCondition = function ( name, evaluator )
		local condition = btnNew ( name, "CONDITION" )
		condition:BTN_SetEvaluator ( evaluator )

		return condition
	end

	return {
		bt_currentNode_,
		bt_node_,
		bt_objID_ = objID,

		BT_Update = bt_update,
		BT_CreateSelector = bt_createSelector,
		BT_CreateSequence = bt_createSequence,
		BT_CreateAction = bt_createAction,
		BT_CreateCondition = bt_createCondition
	}
end